"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const hrtime_now_1 = require("hrtime-now");
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
/**
# Description

Suggests declaring an empty class or interface when a something is extending or implementing something that is not found

# Example

```
class Onion extends NonExistent implements NonExistentInterface{}
```

# Attacks
 
    "code": "2304",	"message": "Cannot find name 'NonExistentInterface'.",

# TODO
 
 * issue: move scroll to end of file
 * issue - break  target class' jsdoc
 * TODO:  modify the file using  insertClass or insertInterface using Structures no text
 * config

 */
exports.declareClass = {
    name: 'declareClass',
    config: {
        // TODO: declare it in new file ? only if user put a special comment ? 
        inNewFile: false,
        // TODO: could be true|false|string . add jsdoc to new class/interface declaration
        jsdoc: true,
    },
    predicate: (arg) => {
        if (arg.containingTargetLight.kind === ts.SyntaxKind.Identifier &&
            arg.diagnostics.find(d => d.code === 2304 && d.start === arg.containingTargetLight.getStart())) {
            return true;
        }
        else {
            arg.log('predicate false because child.kind dont match ' + typescript_ast_util_1.getKindName(arg.containingTargetLight.kind));
            return false;
        }
    },
    description: (arg) => {
        return `Declare type "${arg.containingTargetLight.getText()}"`;
    },
    apply: (arg) => {
        const t0 = hrtime_now_1.now();
        let simpleClassDec = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.ClassDeclaration);
        if (!simpleClassDec) {
            simpleClassDec = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.InterfaceDeclaration);
        }
        if (!simpleClassDec) {
            arg.log('not applying cause cannot find any ancestor of kind ClassDeclaration|InterfaceDeclaration');
        }
        let h;
        if (!ts_simple_ast_1.TypeGuards.isHeritageClause(h = arg.simpleNode.getFirstAncestorByKind(ts.SyntaxKind.HeritageClause))) {
            return;
        }
        const what = simpleClassDec.getKind() === ts.SyntaxKind.InterfaceDeclaration ? 'interface' : h.getToken() === ts.SyntaxKind.ImplementsKeyword ? 'interface' : 'class';
        const code = `
/**
 * TODO: document this ${what}.
 */
${simpleClassDec.isExported() ? 'export ' : ''}${what} ${arg.simpleNode.getText()} {

}
`;
        // TODO:  modify the file using  insertClass or insertInterface using Structures no text
        arg.simpleNode.getSourceFile().insertText(simpleClassDec.getStart(), code);
        arg.log('apply took ' + hrtime_now_1.timeFrom(t0));
    }
};
//# sourceMappingURL=declareClass.js.map