"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
const ts = __importStar(require("typescript"));
const typescript_ast_util_1 = require("typescript-ast-util");
const util_1 = require("../util");
/**

# Description

utilities related to strings:

 * change string quote character
 * go from normal string concatenation to template strings and viceversa
 *
 
# Example

# TODO

 * TODO: still missing the part that transform from string concat to string templates
 * TaggedTemplateExpression getExpression and remove tag - has some utilitiesfor working with rxjs - perhaps we want to include those here?
 * TODO: recurse on expressions that have templateExpressions descendants
 * allow me to convert string concat (a binaryoperationexpression into a single templateExpression
 * config
*/
class Template2Literal {
    constructor() {
        this.name = 'template2Literal';
        this.config = {
            // TODO: so we also transform all expressions recursively from/to templateexpressions - stringliterals
            recurseOnTemplateExpressions: false,
            wrapStringConcatenationWithParenthesis: false
        };
        this.exprBuffer = [];
    }
    predicate(arg) {
        this.action = undefined;
        const bin = typescript_ast_util_1.findAscendant(arg.containingTargetLight, ts.isBinaryExpression, true);
        const binToken = bin && bin.operatorToken.getText();
        // heads up we can change just one literal to template or two operands in a binary expression '+' as long as one of them is a string
        if (ts.isStringLiteral(arg.containingTargetLight) && binToken !== '+') {
            // this.action = 'changeToTemplate'  // TODO: not implemented yet so we are not offering yet
        }
        else if (ts.isNoSubstitutionTemplateLiteral(arg.containingTargetLight) ||
            typescript_ast_util_1.findAscendant(arg.containingTargetLight, ts.isTemplateExpression, true)) {
            this.action = 'changeToLiteral';
        }
        else if (binToken === '+') {
            const tc = arg.program.getTypeChecker();
            if (this.isString(bin.left, tc) || this.isString(bin.right, tc)) {
                this.action = 'changeConcatenationToTemplate'; // TODO: not implemented yet so we are not offering yet
            }
        }
        return !!this.action;
    }
    description(arg) {
        return this.action === 'changeToLiteral' ? `Change to string literal` : this.action === 'changeToTemplate' ? `Change string to template` : `Change concatenation to template`;
    }
    apply(arg) {
        const node = arg.simpleNode;
        if (this.action === 'changeToLiteral') {
            const templateExpr = ts_simple_ast_1.TypeGuards.isNoSubstitutionTemplateLiteral(node) || ts_simple_ast_1.TypeGuards.isTemplateExpression(node) ? node : node.getFirstAncestorByKind(ts.SyntaxKind.TemplateExpression);
            if (templateExpr && ts_simple_ast_1.TypeGuards.isTemplateExpression(templateExpr)) {
                this.templateExprToStringConcat(templateExpr, '"', arg.log);
            }
            else {
                arg.log('apply aborted - target node was not a string literal nor a stringtemplate expression or descendant. target node is ' + node.getText() + ' - kind: ' + node.getKindName());
            }
        }
        if (this.action === 'changeConcatenationToTemplate') {
            const tc = arg.simpleProject.getProgram().getTypeChecker();
            const innerStringConcatExpr = node.getAncestors()
                .find(a => ts_simple_ast_1.TypeGuards.isBinaryExpression(a) && a.getOperatorToken().getText() === '+' && (this.isString2(a.getLeft(), tc) || this.isString2(a.getRight(), tc)));
            if (!innerStringConcatExpr) {
                arg.log('changeConcatenationToTemplate aborted - no outerStringConcatExpression found');
                return;
            }
            this.stringConcatExpr2TemplateRecursively(innerStringConcatExpr, tc);
        }
        else {
            arg.log('action not implemented yet : ' + this.action);
        }
    }
    templateExprToStringConcat(expr, q, log = () => { }) {
        let text;
        if (ts_simple_ast_1.TypeGuards.isNoSubstitutionTemplateLiteral(expr)) {
            util_1.changeQuoteChar(expr, '\''); //TODO: default quotes in case of NoSubstitutionTemplateLiteral ? 
            return;
        }
        const arr = [];
        arr.push(util_1.quote(expr.getHead().getLiteralText(), q));
        expr.getTemplateSpans().forEach(span => {
            // TODO: recurse on expressions that have templateExpressions
            // TODO: only wrap in paren if there is more than one descentant expression
            arr.push('(' + span.getExpression().getText() + ')');
            arr.push(util_1.quote(span.getLiteral().getLiteralText(), q));
        });
        text = (this.config.wrapStringConcatenationWithParenthesis ? '( ' : '') + arr.join(' + ') + (this.config.wrapStringConcatenationWithParenthesis ? ' )' : '');
        log('replacing with text : ' + text);
        expr.replaceWithText(text);
    }
    stringConcatExpr2TemplateRecursively(exprBase, tc) {
        this.exprBuffer = [];
        let expr = exprBase;
        this.stringConcatExpr2Template(exprBase, tc, true);
        while ((expr = expr.getParent()) && ts_simple_ast_1.TypeGuards.isBinaryExpression(expr)) {
            this.stringConcatExpr2Template(expr, tc);
        }
        const text = `\`${this.exprBuffer.join('')}\``;
        expr.getFirstChildByKind(ts.SyntaxKind.BinaryExpression).replaceWithText(text);
    }
    stringConcatExpr2Template(expr, tc, firstTime = false) {
        //TODO: this shall be recursive - from bottom to top
        const left = expr.getLeft();
        const right = expr.getRight();
        let leftText = expr.getLeft().getText(), rightText = expr.getRight().getText();
        if (ts_simple_ast_1.TypeGuards.isStringLiteral(left)) {
            leftText = left.getLiteralText();
        }
        else {
            leftText = '${' + leftText + '}';
        }
        if (firstTime) {
            this.exprBuffer.push(leftText);
        }
        if (ts_simple_ast_1.TypeGuards.isStringLiteral(right)) {
            rightText = right.getLiteralText();
        }
        else {
            rightText = '${' + rightText + '}';
        }
        this.exprBuffer.push(rightText);
    }
    isString(expr, tc) {
        const t = tc.getTypeAtLocation(expr);
        return t.isStringLiteral() ? true : t.getSymbol() ? !!t.getSymbol().getDeclarations().find(d => tc.getTypeAtLocation(d).isStringLiteral()) : false;
    }
    isString2(expr, tc) {
        const t = tc.getTypeAtLocation(expr);
        return t.isStringLiteral() ? true : t.getSymbol() ? !!t.getSymbol().getDeclarations().find(d => tc.getTypeAtLocation(d).isStringLiteral()) : false;
    }
}
exports.Template2Literal = Template2Literal;
exports.template2Literal = new Template2Literal();
//# sourceMappingURL=template2Literal.js.map